///////////////
//
//  Simple object rotator script for Adobe Atmosphere worlds.
//
//  Rolu, www.roce.org, v.1.0, CR 2001-April-16, LU 2001-May-03
//
//  This script allows you to make objects rotate. It's good
//  for billboard objects, gears, art, you name it. As long as
//  it keeps moving at the same speed, you can use this script.
//
//  For more information, see the documentation that came with
//  this script. If there was no documentation, you can find
//  it at www.roce.org. You may use this script freely, as long
//  as you don't cut out these or other comments.
//
///////////////


///////////////
//
//  Argument parser for Adobe Atmosphere scripts
//
//  Rolu, www.roce.org, v.2.1, CR 2001-April-16, LU 2001-May-02
//
//  This parser parses arguments for a script out of the name of "this",
//  creates an object "this.arg" and creates variables in "this.arg" for
//  every argument. If no argument was found in the name of "this", the
//  values from "this.StandardArguments" are used. See the documentation
//  that came with this parser for more information. If there was no
//  documentation, you can find it at www.roce.org. You may copy this
//  code and use it in your own projects, but you must leave all comments
//  in, including this text.
//
///////////////

// Modify this array to change the standard values.
this.StandardArguments=new Array(
    "file", "sky.aer",
    "xrpm", "0",
    "yrpm", "0.15",
    "zrpm", "0",
    "vectrpm", "0",
    "xvect", "0",
    "yvect", "0",
    "zvect", "0",
    "solid", "yes",
    "transparent", "no"
);

// Start of the parser.
this.ParseArgumentsFrom=function(parsestring, defaults)
{
    var tempobj=new Object(), chopped, name, value, quoted, error=false,
        regname=/^\s*(\w+)\s*=/,
        renst=/^\d/,
        renq=/^\s*(\S*)/,
        req=/^\s*("[^"]*")/,
        reempty=/^\s*$/;

    while(defaults.length>1)
        eval("tempobj."+defaults.shift()+"=\""+defaults.shift()+"\";");

    chopped=parsestring;

    while(reempty(chopped)==null)
    {
        quoted=false;

        if(regname(chopped)==null)
        {
            chat.print("Parse error: expected a variable name at the left part of \"",
                chopped, "\".");
            chopped="";
            error=true;
        }
        else
        {
            name=regname(chopped)[1];

            chopped=RegExp.rightContext;        

            if(renst(name)!=null)
            {
                chat.print("Parse error: argument names can not start with a ",
                    "number (", name, ").");
                error=true;
                chopped="";
            }
            else
            {
                value="";

                if(req(chopped)!=null)
                {
                    quoted=true;

                    while(req(chopped)!=null)
                    {
                        value=value.replace(/"$/, "\\");
                        value+=req(chopped)[1];
                        chopped=RegExp.rightContext;
                    }
                }
                else if(renq(chopped)!=null)
                {
                    value=renq(chopped)[1];
                    chopped=RegExp.rightContext;
                }
                else
                {
                    chat.print("Parse error: expected value at the left part of \"",
                        chopped, "\".");
                    chopped="";
                    error=true;
                }
            }
        }

        value.replace(/\\/g, "\\\\");

        if(quoted&&!error)
            eval("tempobj."+name+"="+value+";");
        else if(!error)
            eval("tempobj."+name+"=\""+value+"\";");
    }

    return tempobj;
}
this.arg=this.ParseArgumentsFrom(this.name, this.StandardArguments);
// End of the argument parser.

if(isNaN(this.arg.xrpm))
{
    chat.print("Simple object rotator script: xrpm is not a number, ",
        "but \"", this.arg.xrpm, "\". It is now set to zero.");
    this.arg.xrpm=0;
}
if(isNaN(this.arg.yrpm))
{
    chat.print("Simple object rotator script: yrpm is not a number, ",
        "but \"", this.arg.yrpm, "\". It is now set to zero.");
    this.arg.yrpm=0;
}
if(isNaN(this.arg.zrpm))
{
    chat.print("Simple object rotator script: zrpm is not a number, ",
        "but \"", this.arg.zrpm, "\". It is now set to zero.");
    this.arg.zrpm=0;
}
if(isNaN(this.arg.xvect))
{
    chat.print("Simple object rotator script: xvect is not a number, ",
        "but \"", this.arg.xvect, "\". It is now set to zero.");
    this.arg.xvect=0;
}
if(isNaN(this.arg.yvect))
{
    chat.print("Simple object rotator script: yvect is not a number, ",
        "but \"", this.arg.yvect, "\". It is now set to zero.");
    this.arg.yvect=0;
}
if(isNaN(this.arg.zvect))
{
    chat.print("Simple object rotator script: zvect is not a number, ",
        "but \"", this.arg.zvect, "\". It is now set to zero.");
    this.arg.zvect=0;
}
if(isNaN(this.arg.vectrpm))
{
    chat.print("Simple object rotator script: vectrpm is not a number, ",
        "but \"", this.arg.vectrpm, "\". It is now set to zero.");
    this.arg.vectrpm=0;
}

if(this.arg.file!="")
{
    this.rotobj=SubWorld(this.arg.file).add();
    this.rotobj.position=this.position;
    if(this.arg.solid=="yes")
        this.rotobj.collide=2;
    if(this.arg.transparent=="yes")
        this.rotobj.transparent=true;

    if(this.arg.xrpm!=0)
        this.xspr=60/this.arg.xrpm;
    if(this.arg.yrpm!=0)
        this.yspr=60/this.arg.yrpm;
    if(this.arg.zrpm!=0)
        this.zspr=60/this.arg.zrpm;
    if(this.arg.vectrpm!=0)
        this.vectspr=60/this.arg.vectrpm;

    this.timestep=function(now, del)
    {
        var xadv, yadv, zadv, vectadv;

        if(this.arg.xrpm!=0)
            xadv=(now%this.xspr)/this.xspr;
        else
            xadv=0;

        if(this.arg.yrpm!=0)
            yadv=(now%this.yspr)/this.yspr;
        else
            yadv=0;

        if(this.arg.zrpm!=0)
            zadv=(now%this.zspr)/this.zspr;
        else
            zadv=0;

        if(this.arg.vectrpm!=0)
            vectadv=(now%this.vectspr)/this.vectspr;
        else
            vectspr=0;

        if(xadv==0)
            xadv=0.0001;  // This is due to a problem in build 41,
        if(yadv==0)
            yadv=0.0001;  // if you use a newer build go to www.roce.org
        if(zadv==0)
            zadv=0.0001;  // to see if there is an update available.
        if(vectadv==0)
            vectadv=0.0001;

        this.rotobj.rotateTo(Rotation('XYZ', xadv*Math.PI*2, yadv*Math.PI*2, zadv*Math.PI*2, Vector(this.arg.xvect, this.arg.yvect, this.arg.zvect), vectadv*Math.PI*2));
    }
    addAnimator(this);
}
else
    chat.print("Simple object rotator script: you have not specified ",
        "a filename. You should do this with the \"file\" argument. ",
        "The current arguments: \"", this.name, "\".");
